@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2002-2009 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 2002-2011 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** All rights reserved.
 **
@@ -30,8 +30,6 @@
 ** ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#include "sfconfig.h"
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -98,6 +96,8 @@ encode_file (const char *infilename, const char *outfilename, int filetype)
 	k = 16 - strlen (outfilename) ;
 	PUT_DOTS (k) ;
 
+	memset (&sfinfo, 0, sizeof (sfinfo)) ;
+
 	if (! (infile = sf_open (infilename, SFM_READ, &sfinfo)))
 	{	printf ("Error : could not open file : %s\n", infilename) ;
 		puts (sf_strerror (NULL)) ;
