@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 1999-2017 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 1999-2019 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -22,6 +22,8 @@
 #include <string.h>
 #if HAVE_UNISTD_H
 #include <unistd.h>
+#else
+#include "sf_unistd.h"
 #endif
 #include <ctype.h>
 #include <math.h>
@@ -33,7 +35,7 @@
 #include "sfendian.h"
 #include "common.h"
 
-#define	INITAL_HEADER_SIZE	256
+#define	INITIAL_HEADER_SIZE	256
 
 /* Allocate and initialize the SF_PRIVATE struct. */
 SF_PRIVATE *
@@ -43,11 +45,11 @@ psf_allocate (void)
 	if ((psf = calloc (1, sizeof (SF_PRIVATE))) == NULL)
 		return	NULL ;
 
-	if ((psf->header.ptr = calloc (1, INITAL_HEADER_SIZE)) == NULL)
+	if ((psf->header.ptr = calloc (1, INITIAL_HEADER_SIZE)) == NULL)
 	{	free (psf) ;
 		return	NULL ;
 		} ;
-	psf->header.len = INITAL_HEADER_SIZE ;
+	psf->header.len = INITIAL_HEADER_SIZE ;
 
 	return psf ;
 } /* psf_allocate */
@@ -55,13 +57,13 @@ psf_allocate (void)
 static int
 psf_bump_header_allocation (SF_PRIVATE * psf, sf_count_t needed)
 {
-	sf_count_t newlen, smallest = INITAL_HEADER_SIZE ;
+	sf_count_t newlen, smallest = INITIAL_HEADER_SIZE ;
 	void * ptr ;
 
 	newlen = (needed > psf->header.len) ? 2 * SF_MAX (needed, smallest) : 2 * psf->header.len ;
 
 	if (newlen > 100 * 1024)
-	{	psf_log_printf (psf, "Request for header allocation of %D denined.\n", newlen) ;
+	{	psf_log_printf (psf, "Request for header allocation of %D denied.\n", newlen) ;
 		return 1 ;
 		}
 
@@ -71,6 +73,10 @@ psf_bump_header_allocation (SF_PRIVATE * psf, sf_count_t needed)
 		return 1 ;
 		} ;
 
+	/* Always zero-out new header memory to avoid un-initializer memory accesses. */
+	if (newlen > psf->header.len)
+		memset ((char *) ptr + psf->header.len, 0, newlen - psf->header.len) ;
+
 	psf->header.ptr = ptr ;
 	psf->header.len = newlen ;
 	return 0 ;
@@ -379,6 +385,9 @@ psf_asciiheader_printf (SF_PRIVATE *psf, const char *format, ...)
 	int		maxlen ;
 	char	*start ;
 
+	if (! format)
+		return ;
+
 	maxlen = strlen ((char*) psf->header.ptr) ;
 	start	= ((char*) psf->header.ptr) + maxlen ;
 	maxlen	= psf->header.len - maxlen ;
@@ -552,9 +561,10 @@ psf_binheader_writef (SF_PRIVATE *psf, const char *format, ...)
 	void			*bindata ;
 	size_t			size ;
 	char			c, *strptr ;
-	int				count = 0, trunc_8to4 ;
+	int				count = 0, trunc_8to4 = SF_FALSE ;
 
-	trunc_8to4 = SF_FALSE ;
+	if (! format)
+		return psf_ftell (psf) ;
 
 	va_start (argptr, format) ;
 
@@ -675,16 +685,16 @@ psf_binheader_writef (SF_PRIVATE *psf, const char *format, ...)
 					/* Write a C string (guaranteed to have a zero terminator). */
 					strptr = va_arg (argptr, char *) ;
 					size = strlen (strptr) + 1 ;
-					size += (size & 1) ;
 
-					if (psf->header.indx + (sf_count_t) size >= psf->header.len && psf_bump_header_allocation (psf, 16))
+					if (psf->header.indx + 4 + (sf_count_t) size + (sf_count_t) (size & 1) > psf->header.len && psf_bump_header_allocation (psf, 4 + size + (size & 1)))
 						return count ;
 
 					if (psf->rwf_endian == SF_ENDIAN_BIG)
-						header_put_be_int (psf, size) ;
+						header_put_be_int (psf, size + (size & 1)) ;
 					else
-						header_put_le_int (psf, size) ;
+						header_put_le_int (psf, size + (size & 1)) ;
 					memcpy (&(psf->header.ptr [psf->header.indx]), strptr, size) ;
+					size += (size & 1) ;
 					psf->header.indx += size ;
 					psf->header.ptr [psf->header.indx - 1] = 0 ;
 					count += 4 + size ;
@@ -697,16 +707,15 @@ psf_binheader_writef (SF_PRIVATE *psf, const char *format, ...)
 					*/
 					strptr = va_arg (argptr, char *) ;
 					size = strlen (strptr) ;
-					if (psf->header.indx + (sf_count_t) size > psf->header.len && psf_bump_header_allocation (psf, size))
+					if (psf->header.indx + 4 + (sf_count_t) size + (sf_count_t) (size & 1) > psf->header.len && psf_bump_header_allocation (psf, 4 + size + (size & 1)))
 						return count ;
 					if (psf->rwf_endian == SF_ENDIAN_BIG)
 						header_put_be_int (psf, size) ;
 					else
 						header_put_le_int (psf, size) ;
-					memcpy (&(psf->header.ptr [psf->header.indx]), strptr, size + 1) ;
+					memcpy (&(psf->header.ptr [psf->header.indx]), strptr, size + (size & 1)) ;
 					size += (size & 1) ;
 					psf->header.indx += size ;
-					psf->header.ptr [psf->header.indx] = 0 ;
 					count += 4 + size ;
 					break ;
 
@@ -718,7 +727,7 @@ psf_binheader_writef (SF_PRIVATE *psf, const char *format, ...)
 					size = (size & 1) ? size : size + 1 ;
 					size = (size > 254) ? 254 : size ;
 
-					if (psf->header.indx + (sf_count_t) size > psf->header.len && psf_bump_header_allocation (psf, size))
+					if (psf->header.indx + 1 + (sf_count_t) size > psf->header.len && psf_bump_header_allocation (psf, 1 + size))
 						return count ;
 
 					header_put_byte (psf, size) ;
@@ -868,6 +877,7 @@ header_seek (SF_PRIVATE *psf, sf_count_t position, int whence)
 				psf_bump_header_allocation (psf, position) ;
 			if (position > psf->header.len)
 			{	/* Too much header to cache so just seek instead. */
+				psf->header.indx = psf->header.end ;
 				psf_fseek (psf, position, whence) ;
 				return ;
 				} ;
@@ -1238,8 +1248,12 @@ psf_memset (void *s, int c, sf_count_t len)
 
 typedef SF_CUES_VAR (0) SF_CUES_0 ;
 
+/* calculate size of SF_CUES struct given number of cues */
 #define SF_CUES_VAR_SIZE(count)	(sizeof (SF_CUES_0) + count * sizeof (SF_CUE_POINT))
 
+/* calculate number of cues in SF_CUES struct given data size */
+#define SF_CUES_COUNT(datasize) (((datasize) - sizeof (uint32_t)) / sizeof (SF_CUE_POINT))
+
 SF_CUES *
 psf_cues_alloc (uint32_t cue_count)
 {	SF_CUES *pcues = calloc (1, SF_CUES_VAR_SIZE (cue_count)) ;
@@ -1249,11 +1263,16 @@ psf_cues_alloc (uint32_t cue_count)
 } /* psf_cues_alloc */
 
 SF_CUES *
-psf_cues_dup (const void * ptr)
+psf_cues_dup (const void * ptr, size_t datasize)
 {	const SF_CUES *pcues = ptr ;
-	SF_CUES *pnew = psf_cues_alloc (pcues->cue_count) ;
+	SF_CUES *pnew = NULL ;
+
+	if (pcues->cue_count <= SF_CUES_COUNT (datasize))
+	{	/* check that passed-in datasize is consistent with cue_count in passed-in SF_CUES struct */
+		pnew = psf_cues_alloc (pcues->cue_count) ;
+		memcpy (pnew, pcues, SF_CUES_VAR_SIZE (pcues->cue_count)) ;
+	}
 
-	memcpy (pnew, pcues, SF_CUES_VAR_SIZE (pcues->cue_count)) ;
 	return pnew ;
 } /* psf_cues_dup */
 
@@ -1261,7 +1280,7 @@ void
 psf_get_cues (SF_PRIVATE * psf, void * data, size_t datasize)
 {
 	if (psf->cues)
-	{	uint32_t cue_count = (datasize - sizeof (uint32_t)) / sizeof (SF_CUE_POINT) ;
+	{	uint32_t cue_count = SF_CUES_COUNT (datasize) ;
 
 		cue_count = SF_MIN (cue_count, psf->cues->cue_count) ;
 		memcpy (data, psf->cues, SF_CUES_VAR_SIZE (cue_count)) ;
@@ -1531,6 +1550,9 @@ str_of_minor_format (int format)
 		CASE_NAME (SF_FORMAT_MS_ADPCM) ;
 		CASE_NAME (SF_FORMAT_GSM610) ;
 		CASE_NAME (SF_FORMAT_VOX_ADPCM) ;
+		CASE_NAME (SF_FORMAT_NMS_ADPCM_16) ;
+		CASE_NAME (SF_FORMAT_NMS_ADPCM_24) ;
+		CASE_NAME (SF_FORMAT_NMS_ADPCM_32) ;
 		CASE_NAME (SF_FORMAT_G721_32) ;
 		CASE_NAME (SF_FORMAT_G723_24) ;
 		CASE_NAME (SF_FORMAT_G723_40) ;
@@ -1585,7 +1607,7 @@ psf_f2s_array (const float *src, short *dest, int count, int normalize)
 
 	normfact = normalize ? (1.0 * 0x7FFF) : 1.0 ;
 	while (--count >= 0)
-		dest [count] = lrintf (src [count] * normfact) ;
+		dest [count] = psf_lrintf (src [count] * normfact) ;
 
 	return ;
 } /* psf_f2s_array */
@@ -1607,7 +1629,7 @@ psf_f2s_clip_array (const float *src, short *dest, int count, int normalize)
 			continue ;
 			} ;
 
-		dest [count] = lrintf (scaled_value) ;
+		dest [count] = psf_lrintf (scaled_value) ;
 		} ;
 
 	return ;
@@ -1619,7 +1641,7 @@ psf_d2s_array (const double *src, short *dest, int count, int normalize)
 
 	normfact = normalize ? (1.0 * 0x7FFF) : 1.0 ;
 	while (--count >= 0)
-		dest [count] = lrint (src [count] * normfact) ;
+		dest [count] = psf_lrint (src [count] * normfact) ;
 
 	return ;
 } /* psf_f2s_array */
@@ -1641,7 +1663,7 @@ psf_d2s_clip_array (const double *src, short *dest, int count, int normalize)
 			continue ;
 			} ;
 
-		dest [count] = lrint (scaled_value) ;
+		dest [count] = psf_lrint (scaled_value) ;
 		} ;
 
 	return ;
@@ -1654,7 +1676,7 @@ psf_f2i_array (const float *src, int *dest, int count, int normalize)
 
 	normfact = normalize ? (1.0 * 0x7FFFFFFF) : 1.0 ;
 	while (--count >= 0)
-		dest [count] = lrintf (src [count] * normfact) ;
+		dest [count] = psf_lrintf (src [count] * normfact) ;
 
 	return ;
 } /* psf_f2i_array */
@@ -1676,7 +1698,7 @@ psf_f2i_clip_array (const float *src, int *dest, int count, int normalize)
 			continue ;
 			} ;
 
-		dest [count] = lrintf (scaled_value) ;
+		dest [count] = psf_lrintf (scaled_value) ;
 		} ;
 
 	return ;
@@ -1688,7 +1710,7 @@ psf_d2i_array (const double *src, int *dest, int count, int normalize)
 
 	normfact = normalize ? (1.0 * 0x7FFFFFFF) : 1.0 ;
 	while (--count >= 0)
-		dest [count] = lrint (src [count] * normfact) ;
+		dest [count] = psf_lrint (src [count] * normfact) ;
 
 	return ;
 } /* psf_f2i_array */
@@ -1710,7 +1732,7 @@ psf_d2i_clip_array (const double *src, int *dest, int count, int normalize)
 			continue ;
 			} ;
 
-		dest [count] = lrint (scaled_value) ;
+		dest [count] = psf_lrint (scaled_value) ;
 		} ;
 
 	return ;
