@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 1999-2005 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 1999-2015 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -16,16 +16,17 @@
 ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
+#include	"sfconfig.h"
 
 #include	<stdio.h>
 #include	<stdlib.h>
 #include	<string.h>
+#include	<limits.h>
+#include	<math.h>
 
 #include	"sndfile.h"
-#include	"sfconfig.h"
 #include	"sfendian.h"
 #include	"common.h"
-#include	"float_cast.h"
 
 #if CPU_IS_LITTLE_ENDIAN
 	#define DOUBLE64_READ	double64_le_read
@@ -90,14 +91,19 @@ int
 double64_init	(SF_PRIVATE *psf)
 {	static int double64_caps ;
 
+	if (psf->sf.channels < 1 || psf->sf.channels > SF_MAX_CHANNELS)
+	{	psf_log_printf (psf, "double64_init : internal error : channels = %d\n", psf->sf.channels) ;
+		return SFE_INTERNAL ;
+		} ;
+
 	double64_caps = double64_get_capability (psf) ;
 
 	psf->blockwidth = sizeof (double) * psf->sf.channels ;
 
-	if (psf->mode == SFM_READ || psf->mode == SFM_RDWR)
+	if (psf->file.mode == SFM_READ || psf->file.mode == SFM_RDWR)
 	{	switch (psf->endian + double64_caps)
 		{	case (SF_ENDIAN_BIG + DOUBLE_CAN_RW_BE) :
-					psf->float_endswap = SF_FALSE ;
+					psf->data_endswap = SF_FALSE ;
 					psf->read_short		= host_read_d2s ;
 					psf->read_int		= host_read_d2i ;
 					psf->read_float		= host_read_d2f ;
@@ -105,7 +111,7 @@ double64_init	(SF_PRIVATE *psf)
 					break ;
 
 			case (SF_ENDIAN_LITTLE + DOUBLE_CAN_RW_LE) :
-					psf->float_endswap = SF_FALSE ;
+					psf->data_endswap = SF_FALSE ;
 					psf->read_short		= host_read_d2s ;
 					psf->read_int		= host_read_d2i ;
 					psf->read_float		= host_read_d2f ;
@@ -113,7 +119,7 @@ double64_init	(SF_PRIVATE *psf)
 					break ;
 
 			case (SF_ENDIAN_BIG + DOUBLE_CAN_RW_LE) :
-					psf->float_endswap = SF_TRUE ;
+					psf->data_endswap = SF_TRUE ;
 					psf->read_short		= host_read_d2s ;
 					psf->read_int		= host_read_d2i ;
 					psf->read_float		= host_read_d2f ;
@@ -121,7 +127,7 @@ double64_init	(SF_PRIVATE *psf)
 					break ;
 
 			case (SF_ENDIAN_LITTLE + DOUBLE_CAN_RW_BE) :
-					psf->float_endswap = SF_TRUE ;
+					psf->data_endswap = SF_TRUE ;
 					psf->read_short		= host_read_d2s ;
 					psf->read_int		= host_read_d2i ;
 					psf->read_float		= host_read_d2f ;
@@ -130,7 +136,7 @@ double64_init	(SF_PRIVATE *psf)
 
 			/* When the CPU is not IEEE compatible. */
 			case (SF_ENDIAN_BIG + DOUBLE_BROKEN_BE) :
-					psf->float_endswap = SF_FALSE ;
+					psf->data_endswap = SF_FALSE ;
 					psf->read_short		= replace_read_d2s ;
 					psf->read_int		= replace_read_d2i ;
 					psf->read_float		= replace_read_d2f ;
@@ -138,7 +144,7 @@ double64_init	(SF_PRIVATE *psf)
 					break ;
 
 			case (SF_ENDIAN_LITTLE + DOUBLE_BROKEN_LE) :
-					psf->float_endswap = SF_FALSE ;
+					psf->data_endswap = SF_FALSE ;
 					psf->read_short		= replace_read_d2s ;
 					psf->read_int		= replace_read_d2i ;
 					psf->read_float		= replace_read_d2f ;
@@ -146,7 +152,7 @@ double64_init	(SF_PRIVATE *psf)
 					break ;
 
 			case (SF_ENDIAN_BIG + DOUBLE_BROKEN_LE) :
-					psf->float_endswap = SF_TRUE ;
+					psf->data_endswap = SF_TRUE ;
 					psf->read_short		= replace_read_d2s ;
 					psf->read_int		= replace_read_d2i ;
 					psf->read_float		= replace_read_d2f ;
@@ -154,7 +160,7 @@ double64_init	(SF_PRIVATE *psf)
 					break ;
 
 			case (SF_ENDIAN_LITTLE + DOUBLE_BROKEN_BE) :
-					psf->float_endswap = SF_TRUE ;
+					psf->data_endswap = SF_TRUE ;
 					psf->read_short		= replace_read_d2s ;
 					psf->read_int		= replace_read_d2i ;
 					psf->read_float		= replace_read_d2f ;
@@ -165,10 +171,10 @@ double64_init	(SF_PRIVATE *psf)
 			} ;
 		} ;
 
-	if (psf->mode == SFM_WRITE || psf->mode == SFM_RDWR)
+	if (psf->file.mode == SFM_WRITE || psf->file.mode == SFM_RDWR)
 	{	switch (psf->endian + double64_caps)
 		{	case (SF_ENDIAN_LITTLE + DOUBLE_CAN_RW_LE) :
-					psf->float_endswap = SF_FALSE ;
+					psf->data_endswap = SF_FALSE ;
 					psf->write_short	= host_write_s2d ;
 					psf->write_int		= host_write_i2d ;
 					psf->write_float	= host_write_f2d ;
@@ -176,7 +182,7 @@ double64_init	(SF_PRIVATE *psf)
 					break ;
 
 			case (SF_ENDIAN_BIG + DOUBLE_CAN_RW_BE) :
-					psf->float_endswap = SF_FALSE ;
+					psf->data_endswap = SF_FALSE ;
 					psf->write_short	= host_write_s2d ;
 					psf->write_int		= host_write_i2d ;
 					psf->write_float	= host_write_f2d ;
@@ -184,7 +190,7 @@ double64_init	(SF_PRIVATE *psf)
 					break ;
 
 			case (SF_ENDIAN_BIG + DOUBLE_CAN_RW_LE) :
-					psf->float_endswap = SF_TRUE ;
+					psf->data_endswap = SF_TRUE ;
 					psf->write_short	= host_write_s2d ;
 					psf->write_int		= host_write_i2d ;
 					psf->write_float	= host_write_f2d ;
@@ -192,7 +198,7 @@ double64_init	(SF_PRIVATE *psf)
 					break ;
 
 			case (SF_ENDIAN_LITTLE + DOUBLE_CAN_RW_BE) :
-					psf->float_endswap = SF_TRUE ;
+					psf->data_endswap = SF_TRUE ;
 					psf->write_short	= host_write_s2d ;
 					psf->write_int		= host_write_i2d ;
 					psf->write_float	= host_write_f2d ;
@@ -201,7 +207,7 @@ double64_init	(SF_PRIVATE *psf)
 
 			/* When the CPU is not IEEE compatible. */
 			case (SF_ENDIAN_LITTLE + DOUBLE_BROKEN_LE) :
-					psf->float_endswap = SF_FALSE ;
+					psf->data_endswap = SF_FALSE ;
 					psf->write_short	= replace_write_s2d ;
 					psf->write_int		= replace_write_i2d ;
 					psf->write_float	= replace_write_f2d ;
@@ -209,7 +215,7 @@ double64_init	(SF_PRIVATE *psf)
 					break ;
 
 			case (SF_ENDIAN_BIG + DOUBLE_BROKEN_BE) :
-					psf->float_endswap = SF_FALSE ;
+					psf->data_endswap = SF_FALSE ;
 					psf->write_short	= replace_write_s2d ;
 					psf->write_int		= replace_write_i2d ;
 					psf->write_float	= replace_write_f2d ;
@@ -217,7 +223,7 @@ double64_init	(SF_PRIVATE *psf)
 					break ;
 
 			case (SF_ENDIAN_BIG + DOUBLE_BROKEN_LE) :
-					psf->float_endswap = SF_TRUE ;
+					psf->data_endswap = SF_TRUE ;
 					psf->write_short	= replace_write_s2d ;
 					psf->write_int		= replace_write_i2d ;
 					psf->write_float	= replace_write_f2d ;
@@ -225,7 +231,7 @@ double64_init	(SF_PRIVATE *psf)
 					break ;
 
 			case (SF_ENDIAN_LITTLE + DOUBLE_BROKEN_BE) :
-					psf->float_endswap = SF_TRUE ;
+					psf->data_endswap = SF_TRUE ;
 					psf->write_short	= replace_write_s2d ;
 					psf->write_int		= replace_write_i2d ;
 					psf->write_float	= replace_write_f2d ;
@@ -275,7 +281,7 @@ double64_init	(SF_PRIVATE *psf)
 */
 
 double
-double64_be_read (unsigned char *cptr)
+double64_be_read (const unsigned char *cptr)
 {	int		exponent, negative, upper, lower ;
 	double	dvalue ;
 
@@ -300,15 +306,15 @@ double64_be_read (unsigned char *cptr)
 		dvalue *= -1 ;
 
 	if (exponent > 0)
-		dvalue *= (1 << exponent) ;
+		dvalue *= pow (2.0, exponent) ;
 	else if (exponent < 0)
-		dvalue /= (1 << abs (exponent)) ;
+		dvalue /= pow (2.0, abs (exponent)) ;
 
 	return dvalue ;
 } /* double64_be_read */
 
 double
-double64_le_read (unsigned char *cptr)
+double64_le_read (const unsigned char *cptr)
 {	int		exponent, negative, upper, lower ;
 	double	dvalue ;
 
@@ -333,9 +339,9 @@ double64_le_read (unsigned char *cptr)
 		dvalue *= -1 ;
 
 	if (exponent > 0)
-		dvalue *= (1 << exponent) ;
+		dvalue *= pow (2.0, exponent) ;
 	else if (exponent < 0)
-		dvalue /= (1 << abs (exponent)) ;
+		dvalue /= pow (2.0, abs (exponent)) ;
 
 	return dvalue ;
 } /* double64_le_read */
@@ -362,7 +368,7 @@ double64_be_write (double in, unsigned char *out)
 	out [1] |= (exponent << 4) & 0xF0 ;
 
 	in *= 0x20000000 ;
-	mantissa = lrint (floor (in)) ;
+	mantissa = psf_lrint (floor (in)) ;
 
 	out [1] |= (mantissa >> 24) & 0xF ;
 	out [2] = (mantissa >> 16) & 0xFF ;
@@ -371,7 +377,7 @@ double64_be_write (double in, unsigned char *out)
 
 	in = fmod (in, 1.0) ;
 	in *= 0x1000000 ;
-	mantissa = lrint (floor (in)) ;
+	mantissa = psf_lrint (floor (in)) ;
 
 	out [5] = (mantissa >> 16) & 0xFF ;
 	out [6] = (mantissa >> 8) & 0xFF ;
@@ -402,7 +408,7 @@ double64_le_write (double in, unsigned char *out)
 	out [6] |= (exponent << 4) & 0xF0 ;
 
 	in *= 0x20000000 ;
-	mantissa = lrint (floor (in)) ;
+	mantissa = psf_lrint (floor (in)) ;
 
 	out [6] |= (mantissa >> 24) & 0xF ;
 	out [5] = (mantissa >> 16) & 0xFF ;
@@ -411,7 +417,7 @@ double64_le_write (double in, unsigned char *out)
 
 	in = fmod (in, 1.0) ;
 	in *= 0x1000000 ;
-	mantissa = lrint (floor (in)) ;
+	mantissa = psf_lrint (floor (in)) ;
 
 	out [2] = (mantissa >> 16) & 0xFF ;
 	out [1] = (mantissa >> 8) & 0xFF ;
@@ -452,7 +458,6 @@ static int
 double64_get_capability	(SF_PRIVATE *psf)
 {	union
 	{	double			d ;
-		int				i [2] ;
 		unsigned char	c [8] ;
 	} data ;
 
@@ -460,13 +465,13 @@ double64_get_capability	(SF_PRIVATE *psf)
 
 	if (! psf->ieee_replace)
 	{	/* If this test is true ints and floats are compatible and little endian. */
-		if (data.i [0] == 0x428c59fb && data.i [1] == 0x3ff3c0ca &&
-			data.c [0] == 0xfb && data.c [2] == 0x8c && data.c [4] == 0xca && data.c [6] == 0xf3)
+		if (data.c [0] == 0xfb && data.c [1] == 0x59 && data.c [2] == 0x8c && data.c [3] == 0x42 &&
+			data.c [4] == 0xca && data.c [5] == 0xc0 && data.c [6] == 0xf3 && data.c [7] == 0x3f)
 			return DOUBLE_CAN_RW_LE ;
 
 		/* If this test is true ints and floats are compatible and big endian. */
-		if ((data.i [0] == 0x3ff3c0ca && data.i [1] == 0x428c59fb) &&
-			(data.c [0] == 0x3f && data.c [2] == 0xc0 && data.c [4] == 0x42 && data.c [6] == 0x59))
+		if (data.c [0] == 0x3f && data.c [1] == 0xf3 && data.c [2] == 0xc0 && data.c [3] == 0xca &&
+			data.c [4] == 0x42 && data.c [5] == 0x8c && data.c [6] == 0x59 && data.c [7] == 0xfb)
 			return DOUBLE_CAN_RW_BE ;
 		} ;
 
@@ -479,20 +484,48 @@ double64_get_capability	(SF_PRIVATE *psf)
 /*=======================================================================================
 */
 
-static inline void
+static void
 d2s_array (const double *src, int count, short *dest, double scale)
 {	while (--count >= 0)
-	{	dest [count] = lrint (scale * src [count]) ;
+	{	dest [count] = psf_lrint (scale * src [count]) ;
 		} ;
 } /* d2s_array */
 
-static inline void
+static void
+d2s_clip_array (const double *src, int count, short *dest, double scale)
+{	while (--count >= 0)
+	{	double tmp = scale * src [count] ;
+
+		if (CPU_CLIPS_POSITIVE == 0 && tmp > 32767.0)
+			dest [count] = SHRT_MAX ;
+		else if (CPU_CLIPS_NEGATIVE == 0 && tmp < -32768.0)
+			dest [count] = SHRT_MIN ;
+		else
+			dest [count] = psf_lrint (tmp) ;
+		} ;
+} /* d2s_clip_array */
+
+static void
 d2i_array (const double *src, int count, int *dest, double scale)
 {	while (--count >= 0)
-	{	dest [count] = lrint (scale * src [count]) ;
+	{	dest [count] = psf_lrint (scale * src [count]) ;
 		} ;
 } /* d2i_array */
 
+static void
+d2i_clip_array (const double *src, int count, int *dest, double scale)
+{	while (--count >= 0)
+	{	float tmp = scale * src [count] ;
+
+		if (CPU_CLIPS_POSITIVE == 0 && tmp > (1.0 * INT_MAX))
+			dest [count] = INT_MAX ;
+		else if (CPU_CLIPS_NEGATIVE == 0 && tmp < (-1.0 * INT_MAX))
+			dest [count] = INT_MIN ;
+		else
+			dest [count] = psf_lrint (tmp) ;
+		} ;
+} /* d2i_clip_array */
+
 static inline void
 d2f_array (const double *src, int count, float *dest)
 {	while (--count >= 0)
@@ -501,16 +534,16 @@ d2f_array (const double *src, int count, float *dest)
 } /* d2f_array */
 
 static inline void
-s2d_array (const short *src, double *dest, int count)
+s2d_array (const short *src, double *dest, int count, double scale)
 {	while (--count >= 0)
-	{	dest [count] = src [count] ;
+	{	dest [count] = scale * src [count] ;
 		} ;
 } /* s2d_array */
 
 static inline void
-i2d_array (const int *src, double *dest, int count)
+i2d_array (const int *src, double *dest, int count, double scale)
 {	while (--count >= 0)
-	{	dest [count] = src [count] ;
+	{	dest [count] = scale * src [count] ;
 		} ;
 } /* i2d_array */
 
@@ -526,22 +559,25 @@ f2d_array (const float *src, double *dest, int count)
 
 static sf_count_t
 host_read_d2s	(SF_PRIVATE *psf, short *ptr, sf_count_t len)
-{	int			bufferlen, readcount ;
+{	BUF_UNION	ubuf ;
+	void		(*convert) (const double *, int, short *, double) ;
+	int			bufferlen, readcount ;
 	sf_count_t	total = 0 ;
 	double		scale ;
 
-	bufferlen = ARRAY_LEN (psf->u.dbuf) ;
+	convert = (psf->add_clipping) ? d2s_clip_array : d2s_array ;
+	bufferlen = ARRAY_LEN (ubuf.dbuf) ;
 	scale = (psf->float_int_mult == 0) ? 1.0 : 0x7FFF / psf->float_max ;
 
 	while (len > 0)
 	{	if (len < bufferlen)
 			bufferlen = (int) len ;
-		readcount = psf_fread (psf->u.dbuf, sizeof (double), bufferlen, psf) ;
+		readcount = psf_fread (ubuf.dbuf, sizeof (double), bufferlen, psf) ;
 
-		if (psf->float_endswap == SF_TRUE)
-			endswap_double_array (psf->u.dbuf, readcount) ;
+		if (psf->data_endswap == SF_TRUE)
+			endswap_double_array (ubuf.dbuf, readcount) ;
 
-		d2s_array (psf->u.dbuf, readcount, ptr + total, scale) ;
+		convert (ubuf.dbuf, readcount, ptr + total, scale) ;
 		total += readcount ;
 		len -= readcount ;
 		if (readcount < bufferlen)
@@ -553,21 +589,25 @@ host_read_d2s	(SF_PRIVATE *psf, short *ptr, sf_count_t len)
 
 static sf_count_t
 host_read_d2i	(SF_PRIVATE *psf, int *ptr, sf_count_t len)
-{	int			bufferlen, readcount ;
+{	BUF_UNION	ubuf ;
+	void		(*convert) (const double *, int, int *, double) ;
+	int			bufferlen, readcount ;
 	sf_count_t	total = 0 ;
 	double		scale ;
-	bufferlen = ARRAY_LEN (psf->u.dbuf) ;
-	scale = (psf->float_int_mult == 0) ? 1.0 : 0x7FFFFFFF / psf->float_max ;
+
+	convert = (psf->add_clipping) ? d2i_clip_array : d2i_array ;
+	bufferlen = ARRAY_LEN (ubuf.dbuf) ;
+	scale = (psf->float_int_mult == 0) ? 1.0 : 2147483648.0f / psf->float_max ;
 
 	while (len > 0)
 	{	if (len < bufferlen)
 			bufferlen = (int) len ;
-		readcount = psf_fread (psf->u.dbuf, sizeof (double), bufferlen, psf) ;
+		readcount = psf_fread (ubuf.dbuf, sizeof (double), bufferlen, psf) ;
 
-		if (psf->float_endswap == SF_TRUE)
-			endswap_double_array (psf->u.dbuf, bufferlen) ;
+		if (psf->data_endswap == SF_TRUE)
+			endswap_double_array (ubuf.dbuf, bufferlen) ;
 
-		d2i_array (psf->u.dbuf, readcount, ptr + total, scale) ;
+		convert (ubuf.dbuf, readcount, ptr + total, scale) ;
 		total += readcount ;
 		len -= readcount ;
 		if (readcount < bufferlen)
@@ -579,20 +619,21 @@ host_read_d2i	(SF_PRIVATE *psf, int *ptr, sf_count_t len)
 
 static sf_count_t
 host_read_d2f	(SF_PRIVATE *psf, float *ptr, sf_count_t len)
-{	int			bufferlen, readcount ;
+{	BUF_UNION	ubuf ;
+	int			bufferlen, readcount ;
 	sf_count_t	total = 0 ;
 
-	bufferlen = ARRAY_LEN (psf->u.dbuf) ;
+	bufferlen = ARRAY_LEN (ubuf.dbuf) ;
 
 	while (len > 0)
 	{	if (len < bufferlen)
 			bufferlen = (int) len ;
-		readcount = psf_fread (psf->u.dbuf, sizeof (double), bufferlen, psf) ;
+		readcount = psf_fread (ubuf.dbuf, sizeof (double), bufferlen, psf) ;
 
-		if (psf->float_endswap == SF_TRUE)
-			endswap_double_array (psf->u.dbuf, bufferlen) ;
+		if (psf->data_endswap == SF_TRUE)
+			endswap_double_array (ubuf.dbuf, readcount) ;
 
-		d2f_array (psf->u.dbuf, readcount, ptr + total) ;
+		d2f_array (ubuf.dbuf, readcount, ptr + total) ;
 		total += readcount ;
 		len -= readcount ;
 		if (readcount < bufferlen)
@@ -609,7 +650,7 @@ host_read_d	(SF_PRIVATE *psf, double *ptr, sf_count_t len)
 
 	readcount = psf_fread (ptr, sizeof (double), len, psf) ;
 
-	if (psf->float_endswap != SF_TRUE)
+	if (psf->data_endswap != SF_TRUE)
 		return readcount ;
 
 	/* If the read length was sensible, endswap output in one go. */
@@ -634,24 +675,27 @@ host_read_d	(SF_PRIVATE *psf, double *ptr, sf_count_t len)
 
 static sf_count_t
 host_write_s2d	(SF_PRIVATE *psf, const short *ptr, sf_count_t len)
-{	int			bufferlen, writecount ;
+{	BUF_UNION	ubuf ;
+	int			bufferlen, writecount ;
 	sf_count_t	total = 0 ;
+	double		scale ;
 
-	bufferlen = ARRAY_LEN (psf->u.dbuf) ;
+	scale = (psf->scale_int_float == 0) ? 1.0 : 1.0 / 0x8000 ;
+	bufferlen = ARRAY_LEN (ubuf.dbuf) ;
 
 	while (len > 0)
 	{	if (len < bufferlen)
 			bufferlen = (int) len ;
 
-		s2d_array (ptr + total, psf->u.dbuf, bufferlen) ;
+		s2d_array (ptr + total, ubuf.dbuf, bufferlen, scale) ;
 
 		if (psf->peak_info)
-			double64_peak_update (psf, psf->u.dbuf, bufferlen, total / psf->sf.channels) ;
+			double64_peak_update (psf, ubuf.dbuf, bufferlen, total / psf->sf.channels) ;
 
-		if (psf->float_endswap == SF_TRUE)
-			endswap_double_array (psf->u.dbuf, bufferlen) ;
+		if (psf->data_endswap == SF_TRUE)
+			endswap_double_array (ubuf.dbuf, bufferlen) ;
 
-		writecount = psf_fwrite (psf->u.dbuf, sizeof (double), bufferlen, psf) ;
+		writecount = psf_fwrite (ubuf.dbuf, sizeof (double), bufferlen, psf) ;
 		total += writecount ;
 		if (writecount < bufferlen)
 			break ;
@@ -663,23 +707,26 @@ host_write_s2d	(SF_PRIVATE *psf, const short *ptr, sf_count_t len)
 
 static sf_count_t
 host_write_i2d	(SF_PRIVATE *psf, const int *ptr, sf_count_t len)
-{	int			bufferlen, writecount ;
+{	BUF_UNION	ubuf ;
+	int			bufferlen, writecount ;
 	sf_count_t	total = 0 ;
+	double		scale ;
 
-	bufferlen = ARRAY_LEN (psf->u.dbuf) ;
+	scale = (psf->scale_int_float == 0) ? 1.0 : 1.0 / (8.0 * 0x10000000) ;
+	bufferlen = ARRAY_LEN (ubuf.dbuf) ;
 
 	while (len > 0)
 	{	if (len < bufferlen)
 			bufferlen = (int) len ;
-		i2d_array (ptr + total, psf->u.dbuf, bufferlen) ;
+		i2d_array (ptr + total, ubuf.dbuf, bufferlen, scale) ;
 
 		if (psf->peak_info)
-			double64_peak_update (psf, psf->u.dbuf, bufferlen, total / psf->sf.channels) ;
+			double64_peak_update (psf, ubuf.dbuf, bufferlen, total / psf->sf.channels) ;
 
-		if (psf->float_endswap == SF_TRUE)
-			endswap_double_array (psf->u.dbuf, bufferlen) ;
+		if (psf->data_endswap == SF_TRUE)
+			endswap_double_array (ubuf.dbuf, bufferlen) ;
 
-		writecount = psf_fwrite (psf->u.dbuf, sizeof (double), bufferlen, psf) ;
+		writecount = psf_fwrite (ubuf.dbuf, sizeof (double), bufferlen, psf) ;
 		total += writecount ;
 		if (writecount < bufferlen)
 			break ;
@@ -691,23 +738,24 @@ host_write_i2d	(SF_PRIVATE *psf, const int *ptr, sf_count_t len)
 
 static sf_count_t
 host_write_f2d	(SF_PRIVATE *psf, const float *ptr, sf_count_t len)
-{	int			bufferlen, writecount ;
+{	BUF_UNION	ubuf ;
+	int			bufferlen, writecount ;
 	sf_count_t	total = 0 ;
 
-	bufferlen = ARRAY_LEN (psf->u.dbuf) ;
+	bufferlen = ARRAY_LEN (ubuf.dbuf) ;
 
 	while (len > 0)
 	{	if (len < bufferlen)
 			bufferlen = (int) len ;
-		f2d_array (ptr + total, psf->u.dbuf, bufferlen) ;
+		f2d_array (ptr + total, ubuf.dbuf, bufferlen) ;
 
 		if (psf->peak_info)
-			double64_peak_update (psf, psf->u.dbuf, bufferlen, total / psf->sf.channels) ;
+			double64_peak_update (psf, ubuf.dbuf, bufferlen, total / psf->sf.channels) ;
 
-		if (psf->float_endswap == SF_TRUE)
-			endswap_double_array (psf->u.dbuf, bufferlen) ;
+		if (psf->data_endswap == SF_TRUE)
+			endswap_double_array (ubuf.dbuf, bufferlen) ;
 
-		writecount = psf_fwrite (psf->u.dbuf, sizeof (double), bufferlen, psf) ;
+		writecount = psf_fwrite (ubuf.dbuf, sizeof (double), bufferlen, psf) ;
 		total += writecount ;
 		if (writecount < bufferlen)
 			break ;
@@ -719,24 +767,25 @@ host_write_f2d	(SF_PRIVATE *psf, const float *ptr, sf_count_t len)
 
 static sf_count_t
 host_write_d	(SF_PRIVATE *psf, const double *ptr, sf_count_t len)
-{	int			bufferlen, writecount ;
+{	BUF_UNION	ubuf ;
+	int			bufferlen, writecount ;
 	sf_count_t	total = 0 ;
 
 	if (psf->peak_info)
 		double64_peak_update (psf, ptr, len, 0) ;
 
-	if (psf->float_endswap != SF_TRUE)
+	if (psf->data_endswap != SF_TRUE)
 		return psf_fwrite (ptr, sizeof (double), len, psf) ;
 
-	bufferlen = ARRAY_LEN (psf->u.dbuf) ;
+	bufferlen = ARRAY_LEN (ubuf.dbuf) ;
 
 	while (len > 0)
 	{	if (len < bufferlen)
 			bufferlen = (int) len ;
 
-		endswap_double_copy (psf->u.dbuf, ptr + total, bufferlen) ;
+		endswap_double_copy (ubuf.dbuf, ptr + total, bufferlen) ;
 
-		writecount = psf_fwrite (psf->u.dbuf, sizeof (double), bufferlen, psf) ;
+		writecount = psf_fwrite (ubuf.dbuf, sizeof (double), bufferlen, psf) ;
 		total += writecount ;
 		if (writecount < bufferlen)
 			break ;
@@ -751,24 +800,25 @@ host_write_d	(SF_PRIVATE *psf, const double *ptr, sf_count_t len)
 
 static sf_count_t
 replace_read_d2s	(SF_PRIVATE *psf, short *ptr, sf_count_t len)
-{	int			bufferlen, readcount ;
+{	BUF_UNION	ubuf ;
+	int			bufferlen, readcount ;
 	sf_count_t	total = 0 ;
 	double		scale ;
 
-	bufferlen = ARRAY_LEN (psf->u.dbuf) ;
+	bufferlen = ARRAY_LEN (ubuf.dbuf) ;
 	scale = (psf->float_int_mult == 0) ? 1.0 : 0x7FFF / psf->float_max ;
 
 	while (len > 0)
 	{	if (len < bufferlen)
 			bufferlen = (int) len ;
-		readcount = psf_fread (psf->u.dbuf, sizeof (double), bufferlen, psf) ;
+		readcount = psf_fread (ubuf.dbuf, sizeof (double), bufferlen, psf) ;
 
-		if (psf->float_endswap == SF_TRUE)
-			endswap_double_array (psf->u.dbuf, bufferlen) ;
+		if (psf->data_endswap == SF_TRUE)
+			endswap_double_array (ubuf.dbuf, bufferlen) ;
 
-		d2bd_read (psf->u.dbuf, bufferlen) ;
+		d2bd_read (ubuf.dbuf, bufferlen) ;
 
-		d2s_array (psf->u.dbuf, readcount, ptr + total, scale) ;
+		d2s_array (ubuf.dbuf, readcount, ptr + total, scale) ;
 		total += readcount ;
 		if (readcount < bufferlen)
 			break ;
@@ -780,24 +830,25 @@ replace_read_d2s	(SF_PRIVATE *psf, short *ptr, sf_count_t len)
 
 static sf_count_t
 replace_read_d2i	(SF_PRIVATE *psf, int *ptr, sf_count_t len)
-{	int			bufferlen, readcount ;
+{	BUF_UNION	ubuf ;
+	int			bufferlen, readcount ;
 	sf_count_t	total = 0 ;
 	double		scale ;
 
-	bufferlen = ARRAY_LEN (psf->u.dbuf) ;
-	scale = (psf->float_int_mult == 0) ? 1.0 : 0x7FFFFFFF / psf->float_max ;
+	bufferlen = ARRAY_LEN (ubuf.dbuf) ;
+	scale = (psf->float_int_mult == 0) ? 1.0 : 2147483648.0f / psf->float_max ;
 
 	while (len > 0)
 	{	if (len < bufferlen)
 			bufferlen = (int) len ;
-		readcount = psf_fread (psf->u.dbuf, sizeof (double), bufferlen, psf) ;
+		readcount = psf_fread (ubuf.dbuf, sizeof (double), bufferlen, psf) ;
 
-		if (psf->float_endswap == SF_TRUE)
-			endswap_double_array (psf->u.dbuf, bufferlen) ;
+		if (psf->data_endswap == SF_TRUE)
+			endswap_double_array (ubuf.dbuf, bufferlen) ;
 
-		d2bd_read (psf->u.dbuf, bufferlen) ;
+		d2bd_read (ubuf.dbuf, bufferlen) ;
 
-		d2i_array (psf->u.dbuf, readcount, ptr + total, scale) ;
+		d2i_array (ubuf.dbuf, readcount, ptr + total, scale) ;
 		total += readcount ;
 		if (readcount < bufferlen)
 			break ;
@@ -809,22 +860,23 @@ replace_read_d2i	(SF_PRIVATE *psf, int *ptr, sf_count_t len)
 
 static sf_count_t
 replace_read_d2f	(SF_PRIVATE *psf, float *ptr, sf_count_t len)
-{	int			bufferlen, readcount ;
+{	BUF_UNION	ubuf ;
+	int			bufferlen, readcount ;
 	sf_count_t	total = 0 ;
 
-	bufferlen = ARRAY_LEN (psf->u.dbuf) ;
+	bufferlen = ARRAY_LEN (ubuf.dbuf) ;
 
 	while (len > 0)
 	{	if (len < bufferlen)
 			bufferlen = (int) len ;
-		readcount = psf_fread (psf->u.dbuf, sizeof (double), bufferlen, psf) ;
+		readcount = psf_fread (ubuf.dbuf, sizeof (double), bufferlen, psf) ;
 
-		if (psf->float_endswap == SF_TRUE)
-			endswap_double_array (psf->u.dbuf, bufferlen) ;
+		if (psf->data_endswap == SF_TRUE)
+			endswap_double_array (ubuf.dbuf, bufferlen) ;
 
-		d2bd_read (psf->u.dbuf, bufferlen) ;
+		d2bd_read (ubuf.dbuf, bufferlen) ;
 
-		memcpy (ptr + total, psf->u.dbuf, bufferlen * sizeof (double)) ;
+		memcpy (ptr + total, ubuf.dbuf, bufferlen * sizeof (double)) ;
 
 		total += readcount ;
 		if (readcount < bufferlen)
@@ -837,23 +889,24 @@ replace_read_d2f	(SF_PRIVATE *psf, float *ptr, sf_count_t len)
 
 static sf_count_t
 replace_read_d	(SF_PRIVATE *psf, double *ptr, sf_count_t len)
-{	int			bufferlen, readcount ;
+{	BUF_UNION	ubuf ;
+	int			bufferlen, readcount ;
 	sf_count_t	total = 0 ;
 
 	/* FIXME : This is probably nowhere near optimal. */
-	bufferlen = ARRAY_LEN (psf->u.dbuf) ;
+	bufferlen = ARRAY_LEN (ubuf.dbuf) ;
 
 	while (len > 0)
 	{	if (len < bufferlen)
 			bufferlen = (int) len ;
-		readcount = psf_fread (psf->u.dbuf, sizeof (double), bufferlen, psf) ;
+		readcount = psf_fread (ubuf.dbuf, sizeof (double), bufferlen, psf) ;
 
-		if (psf->float_endswap == SF_TRUE)
-			endswap_double_array (psf->u.dbuf, readcount) ;
+		if (psf->data_endswap == SF_TRUE)
+			endswap_double_array (ubuf.dbuf, readcount) ;
 
-		d2bd_read (psf->u.dbuf, readcount) ;
+		d2bd_read (ubuf.dbuf, readcount) ;
 
-		memcpy (ptr + total, psf->u.dbuf, readcount * sizeof (double)) ;
+		memcpy (ptr + total, ubuf.dbuf, readcount * sizeof (double)) ;
 
 		total += readcount ;
 		if (readcount < bufferlen)
@@ -866,25 +919,28 @@ replace_read_d	(SF_PRIVATE *psf, double *ptr, sf_count_t len)
 
 static sf_count_t
 replace_write_s2d	(SF_PRIVATE *psf, const short *ptr, sf_count_t len)
-{	int			bufferlen, writecount ;
+{	BUF_UNION	ubuf ;
+	int			bufferlen, writecount ;
 	sf_count_t	total = 0 ;
+	double		scale ;
 
-	bufferlen = ARRAY_LEN (psf->u.dbuf) ;
+	scale = (psf->scale_int_float == 0) ? 1.0 : 1.0 / 0x8000 ;
+	bufferlen = ARRAY_LEN (ubuf.dbuf) ;
 
 	while (len > 0)
 	{	if (len < bufferlen)
 			bufferlen = (int) len ;
-		s2d_array (ptr + total, psf->u.dbuf, bufferlen) ;
+		s2d_array (ptr + total, ubuf.dbuf, bufferlen, scale) ;
 
 		if (psf->peak_info)
-			double64_peak_update (psf, psf->u.dbuf, bufferlen, total / psf->sf.channels) ;
+			double64_peak_update (psf, ubuf.dbuf, bufferlen, total / psf->sf.channels) ;
 
-		bd2d_write (psf->u.dbuf, bufferlen) ;
+		bd2d_write (ubuf.dbuf, bufferlen) ;
 
-		if (psf->float_endswap == SF_TRUE)
-			endswap_double_array (psf->u.dbuf, bufferlen) ;
+		if (psf->data_endswap == SF_TRUE)
+			endswap_double_array (ubuf.dbuf, bufferlen) ;
 
-		writecount = psf_fwrite (psf->u.dbuf, sizeof (double), bufferlen, psf) ;
+		writecount = psf_fwrite (ubuf.dbuf, sizeof (double), bufferlen, psf) ;
 		total += writecount ;
 		if (writecount < bufferlen)
 			break ;
@@ -896,25 +952,28 @@ replace_write_s2d	(SF_PRIVATE *psf, const short *ptr, sf_count_t len)
 
 static sf_count_t
 replace_write_i2d	(SF_PRIVATE *psf, const int *ptr, sf_count_t len)
-{	int			bufferlen, writecount ;
+{	BUF_UNION	ubuf ;
+	int			bufferlen, writecount ;
 	sf_count_t	total = 0 ;
+	double		scale ;
 
-	bufferlen = ARRAY_LEN (psf->u.dbuf) ;
+	scale = (psf->scale_int_float == 0) ? 1.0 : 1.0 / (8.0 * 0x10000000) ;
+	bufferlen = ARRAY_LEN (ubuf.dbuf) ;
 
 	while (len > 0)
 	{	if (len < bufferlen)
 			bufferlen = (int) len ;
-		i2d_array (ptr + total, psf->u.dbuf, bufferlen) ;
+		i2d_array (ptr + total, ubuf.dbuf, bufferlen, scale) ;
 
 		if (psf->peak_info)
-			double64_peak_update (psf, psf->u.dbuf, bufferlen, total / psf->sf.channels) ;
+			double64_peak_update (psf, ubuf.dbuf, bufferlen, total / psf->sf.channels) ;
 
-		bd2d_write (psf->u.dbuf, bufferlen) ;
+		bd2d_write (ubuf.dbuf, bufferlen) ;
 
-		if (psf->float_endswap == SF_TRUE)
-			endswap_double_array (psf->u.dbuf, bufferlen) ;
+		if (psf->data_endswap == SF_TRUE)
+			endswap_double_array (ubuf.dbuf, bufferlen) ;
 
-		writecount = psf_fwrite (psf->u.dbuf, sizeof (double), bufferlen, psf) ;
+		writecount = psf_fwrite (ubuf.dbuf, sizeof (double), bufferlen, psf) ;
 		total += writecount ;
 		if (writecount < bufferlen)
 			break ;
@@ -926,22 +985,23 @@ replace_write_i2d	(SF_PRIVATE *psf, const int *ptr, sf_count_t len)
 
 static sf_count_t
 replace_write_f2d	(SF_PRIVATE *psf, const float *ptr, sf_count_t len)
-{	int			bufferlen, writecount ;
+{	BUF_UNION	ubuf ;
+	int			bufferlen, writecount ;
 	sf_count_t	total = 0 ;
 
-	bufferlen = ARRAY_LEN (psf->u.dbuf) ;
+	bufferlen = ARRAY_LEN (ubuf.dbuf) ;
 
 	while (len > 0)
 	{	if (len < bufferlen)
 			bufferlen = (int) len ;
-		f2d_array (ptr + total, psf->u.dbuf, bufferlen) ;
+		f2d_array (ptr + total, ubuf.dbuf, bufferlen) ;
 
-		bd2d_write (psf->u.dbuf, bufferlen) ;
+		bd2d_write (ubuf.dbuf, bufferlen) ;
 
-		if (psf->float_endswap == SF_TRUE)
-			endswap_double_array (psf->u.dbuf, bufferlen) ;
+		if (psf->data_endswap == SF_TRUE)
+			endswap_double_array (ubuf.dbuf, bufferlen) ;
 
-		writecount = psf_fwrite (psf->u.dbuf, sizeof (double), bufferlen, psf) ;
+		writecount = psf_fwrite (ubuf.dbuf, sizeof (double), bufferlen, psf) ;
 		total += writecount ;
 		if (writecount < bufferlen)
 			break ;
@@ -953,27 +1013,28 @@ replace_write_f2d	(SF_PRIVATE *psf, const float *ptr, sf_count_t len)
 
 static sf_count_t
 replace_write_d	(SF_PRIVATE *psf, const double *ptr, sf_count_t len)
-{	int			bufferlen, writecount ;
+{	BUF_UNION	ubuf ;
+	int			bufferlen, writecount ;
 	sf_count_t	total = 0 ;
 
 	/* FIXME : This is probably nowhere near optimal. */
 	if (psf->peak_info)
 		double64_peak_update (psf, ptr, len, 0) ;
 
-	bufferlen = ARRAY_LEN (psf->u.dbuf) ;
+	bufferlen = ARRAY_LEN (ubuf.dbuf) ;
 
 	while (len > 0)
 	{	if (len < bufferlen)
 			bufferlen = (int) len ;
 
-		memcpy (psf->u.dbuf, ptr + total, bufferlen * sizeof (double)) ;
+		memcpy (ubuf.dbuf, ptr + total, bufferlen * sizeof (double)) ;
 
-		bd2d_write (psf->u.dbuf, bufferlen) ;
+		bd2d_write (ubuf.dbuf, bufferlen) ;
 
-		if (psf->float_endswap == SF_TRUE)
-			endswap_double_array (psf->u.dbuf, bufferlen) ;
+		if (psf->data_endswap == SF_TRUE)
+			endswap_double_array (ubuf.dbuf, bufferlen) ;
 
-		writecount = psf_fwrite (psf->u.dbuf, sizeof (double), bufferlen, psf) ;
+		writecount = psf_fwrite (ubuf.dbuf, sizeof (double), bufferlen, psf) ;
 		total += writecount ;
 		if (writecount < bufferlen)
 			break ;
@@ -1000,10 +1061,3 @@ bd2d_write (double *buffer, int count)
 		} ;
 } /* bd2d_write */
 
-/*
-** Do not edit or modify anything in this comment block.
-** The arch-tag line is a file identity tag for the GNU Arch
-** revision control system.
-**
-** arch-tag: 4ee243b7-8c7a-469b-869c-e9aa0ee3b77f
-*/
