@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2002-2010 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 2002-2014 Erik de Castro Lopo <erikd@mega-nerd.com>
 ** Copyright (C) 2003 Ross Bencina <rbencina@iprimus.com.au>
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -34,11 +34,21 @@
 
 #include "sfconfig.h"
 
+#if USE_WINDOWS_API
+
+/* Don't include rarely used headers, speed up build */
+#define WIN32_LEAN_AND_MEAN
+
+#include <windows.h>
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 
 #if HAVE_UNISTD_H
 #include <unistd.h>
+#else
+#include <io.h>
 #endif
 
 #if (HAVE_DECL_S_IRGRP == 0)
@@ -116,33 +126,36 @@ psf_fclose (SF_PRIVATE *psf)
 
 int
 psf_open_rsrc (SF_PRIVATE *psf)
-{
+{	size_t count ;
+
 	if (psf->rsrc.filedes > 0)
 		return 0 ;
 
 	/* Test for MacOSX style resource fork on HPFS or HPFS+ filesystems. */
-	snprintf (psf->rsrc.path.c, sizeof (psf->rsrc.path.c), "%s/rsrc", psf->file.path.c) ;
+	count = snprintf (psf->rsrc.path.c, sizeof (psf->rsrc.path.c), "%s/..namedfork/rsrc", psf->file.path.c) ;
 	psf->error = SFE_NO_ERROR ;
-	if ((psf->rsrc.filedes = psf_open_fd (&psf->rsrc)) >= 0)
-	{	psf->rsrclength = psf_get_filelen_fd (psf->rsrc.filedes) ;
-		if (psf->rsrclength > 0 || (psf->rsrc.mode & SFM_WRITE))
-			return SFE_NO_ERROR ;
-		psf_close_fd (psf->rsrc.filedes) ;
-		psf->rsrc.filedes = -1 ;
-		} ;
+	if (count < sizeof (psf->rsrc.path.c))
+	{	if ((psf->rsrc.filedes = psf_open_fd (&psf->rsrc)) >= 0)
+		{	psf->rsrclength = psf_get_filelen_fd (psf->rsrc.filedes) ;
+			if (psf->rsrclength > 0 || (psf->rsrc.mode & SFM_WRITE))
+				return SFE_NO_ERROR ;
+			psf_close_fd (psf->rsrc.filedes) ;
+			psf->rsrc.filedes = -1 ;
+			} ;
 
-	if (psf->rsrc.filedes == - SFE_BAD_OPEN_MODE)
-	{	psf->error = SFE_BAD_OPEN_MODE ;
-		return psf->error ;
+		if (psf->rsrc.filedes == - SFE_BAD_OPEN_MODE)
+		{	psf->error = SFE_BAD_OPEN_MODE ;
+			return psf->error ;
+			} ;
 		} ;
 
 	/*
 	** Now try for a resource fork stored as a separate file in the same
 	** directory, but preceded with a dot underscore.
 	*/
-	snprintf (psf->rsrc.path.c, sizeof (psf->rsrc.path.c), "%s._%s", psf->file.dir.c, psf->file.name.c) ;
+	count = snprintf (psf->rsrc.path.c, sizeof (psf->rsrc.path.c), "%s._%s", psf->file.dir.c, psf->file.name.c) ;
 	psf->error = SFE_NO_ERROR ;
-	if ((psf->rsrc.filedes = psf_open_fd (&psf->rsrc)) >= 0)
+	if (count < sizeof (psf->rsrc.path.c) && (psf->rsrc.filedes = psf_open_fd (&psf->rsrc)) >= 0)
 	{	psf->rsrclength = psf_get_filelen_fd (psf->rsrc.filedes) ;
 		return SFE_NO_ERROR ;
 		} ;
@@ -151,16 +164,21 @@ psf_open_rsrc (SF_PRIVATE *psf)
 	** Now try for a resource fork stored in a separate file in the
 	** .AppleDouble/ directory.
 	*/
-	snprintf (psf->rsrc.path.c, sizeof (psf->rsrc.path.c), "%s.AppleDouble/%s", psf->file.dir.c, psf->file.name.c) ;
+	count = snprintf (psf->rsrc.path.c, sizeof (psf->rsrc.path.c), "%s.AppleDouble/%s", psf->file.dir.c, psf->file.name.c) ;
 	psf->error = SFE_NO_ERROR ;
-	if ((psf->rsrc.filedes = psf_open_fd (&psf->rsrc)) >= 0)
-	{	psf->rsrclength = psf_get_filelen_fd (psf->rsrc.filedes) ;
-		return SFE_NO_ERROR ;
-		} ;
+	if (count < sizeof (psf->rsrc.path.c))
+	{	if ((psf->rsrc.filedes = psf_open_fd (&psf->rsrc)) >= 0)
+		{	psf->rsrclength = psf_get_filelen_fd (psf->rsrc.filedes) ;
+			return SFE_NO_ERROR ;
+			} ;
 
-	/* No resource file found. */
-	if (psf->rsrc.filedes == -1)
-		psf_log_syserr (psf, errno) ;
+		/* No resource file found. */
+		if (psf->rsrc.filedes == -1)
+			psf_log_syserr (psf, errno) ;
+		}
+	else
+	{	psf->error = SFE_OPEN_FAILED ;
+		} ;
 
 	psf->rsrc.filedes = -1 ;
 
@@ -257,12 +275,17 @@ psf_file_valid (SF_PRIVATE *psf)
 
 sf_count_t
 psf_fseek (SF_PRIVATE *psf, sf_count_t offset, int whence)
-{	sf_count_t	current_pos, new_position ;
+{	sf_count_t	absolute_position ;
 
 	if (psf->virtual_io)
 		return psf->vio.seek (offset, whence, psf->vio_user_data) ;
 
-	current_pos = psf_ftell (psf) ;
+	/* When decoding from pipes sometimes see seeks to the pipeoffset, which appears to mean do nothing. */
+	if (psf->is_pipe)
+	{	if (whence != SEEK_SET || offset != psf->pipeoffset)
+			psf_log_printf (psf, "psf_fseek : pipe seek to value other than pipeoffset\n") ;
+		return offset ;
+		}
 
 	switch (whence)
 	{	case SEEK_SET :
@@ -270,27 +293,9 @@ psf_fseek (SF_PRIVATE *psf, sf_count_t offset, int whence)
 				break ;
 
 		case SEEK_END :
-				if (psf->file.mode == SFM_WRITE)
-				{	new_position = lseek (psf->file.filedes, offset, whence) ;
-
-					if (new_position < 0)
-						psf_log_syserr (psf, errno) ;
-
-					return new_position - psf->fileoffset ;
-					} ;
-
-				/* Transform SEEK_END into a SEEK_SET, ie find the file
-				** length add the requested offset (should be <= 0) to
-				** get the offset wrt the start of file.
-				*/
-				whence = SEEK_SET ;
-				offset = lseek (psf->file.filedes, 0, SEEK_END) + offset ;
 				break ;
 
 		case SEEK_CUR :
-				/* Translate a SEEK_CUR into a SEEK_SET. */
-				offset += current_pos ;
-				whence = SEEK_SET ;
 				break ;
 
 		default :
@@ -299,17 +304,12 @@ psf_fseek (SF_PRIVATE *psf, sf_count_t offset, int whence)
 				return 0 ;
 		} ;
 
-	if (current_pos != offset)
-		new_position = lseek (psf->file.filedes, offset, whence) ;
-	else
-		new_position = offset ;
+	absolute_position = lseek (psf->file.filedes, offset, whence) ;
 
-	if (new_position < 0)
+	if (absolute_position < 0)
 		psf_log_syserr (psf, errno) ;
 
-	new_position -= psf->fileoffset ;
-
-	return new_position ;
+	return absolute_position - psf->fileoffset ;
 } /* psf_fseek */
 
 sf_count_t
@@ -358,6 +358,9 @@ psf_fwrite (const void *ptr, sf_count_t bytes, sf_count_t items, SF_PRIVATE *psf
 {	sf_count_t total = 0 ;
 	ssize_t	count ;
 
+	if (bytes == 0 || items == 0)
+		return 0 ;
+
 	if (psf->virtual_io)
 		return psf->vio.write (ptr, bytes*items, psf->vio_user_data) / bytes ;
 
@@ -473,19 +476,22 @@ psf_is_pipe (SF_PRIVATE *psf)
 
 static sf_count_t
 psf_get_filelen_fd (int fd)
-{	struct stat statbuf ;
+{
+#if (SIZEOF_OFF_T == 4 && SIZEOF_SF_COUNT_T == 8 && HAVE_FSTAT64)
+	struct stat64 statbuf ;
 
-	/*
-	** Sanity check.
-	** If everything is OK, this will be optimised out.
-	*/
-	if (sizeof (statbuf.st_size) == 4 && sizeof (sf_count_t) == 8)
-		return (sf_count_t) -SFE_BAD_STAT_SIZE ;
+	if (fstat64 (fd, &statbuf) == -1)
+		return (sf_count_t) -1 ;
+
+	return statbuf.st_size ;
+#else
+	struct stat statbuf ;
 
 	if (fstat (fd, &statbuf) == -1)
 		return (sf_count_t) -1 ;
 
 	return statbuf.st_size ;
+#endif
 } /* psf_get_filelen_fd */
 
 int
@@ -538,8 +544,8 @@ psf_open_fd (PSF_FILE * pfile)
 	** be optimised out. This is meant to catch the problems caused by
 	** "sfconfig.h" being included after <stdio.h>.
 	*/
-	if (sizeof (off_t) != sizeof (sf_count_t))
-	{	puts ("\n\n*** Fatal error : sizeof (off_t) != sizeof (sf_count_t)") ;
+	if (sizeof (sf_count_t) != 8)
+	{	puts ("\n\n*** Fatal error : sizeof (sf_count_t) != 8") ;
 		puts ("*** This means that libsndfile was not configured correctly.\n") ;
 		exit (1) ;
 		} ;
@@ -552,12 +558,12 @@ psf_open_fd (PSF_FILE * pfile)
 
 		case SFM_WRITE :
 				oflag = O_WRONLY | O_CREAT | O_TRUNC | O_BINARY ;
-				mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH ;
+				mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH ;
 				break ;
 
 		case SFM_RDWR :
 				oflag = O_RDWR | O_CREAT | O_BINARY ;
-				mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH ;
+				mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH ;
 				break ;
 
 		default :
@@ -589,10 +595,10 @@ void
 psf_fsync (SF_PRIVATE *psf)
 {
 #if HAVE_FSYNC
-    if (psf->file.mode == SFM_WRITE || psf->file.mode == SFM_RDWR)
-        fsync (psf->file.filedes) ;
+	if (psf->file.mode == SFM_WRITE || psf->file.mode == SFM_RDWR)
+		fsync (psf->file.filedes) ;
 #else
-    psf = NULL ;
+	psf = NULL ;
 #endif
 } /* psf_fsync */
 
@@ -600,8 +606,9 @@ psf_fsync (SF_PRIVATE *psf)
 
 /* Win32 file i/o functions implemented using native Win32 API */
 
-#include <windows.h>
-#include <io.h>
+#ifndef WINAPI_PARTITION_SYSTEM
+#define WINAPI_PARTITION_SYSTEM 0
+#endif
 
 static int psf_close_handle (HANDLE handle) ;
 static HANDLE psf_open_handle (PSF_FILE * pfile) ;
@@ -672,7 +679,7 @@ psf_open_rsrc (SF_PRIVATE *psf)
 	psf->error = SFE_NO_ERROR ;
 	if ((psf->rsrc.handle = psf_open_handle (&psf->rsrc)) != NULL)
 	{	psf->rsrclength = psf_get_filelen_handle (psf->rsrc.handle) ;
-      		return SFE_NO_ERROR ;
+		return SFE_NO_ERROR ;
 		} ;
 
 	/* No resource file found. */
@@ -781,6 +788,21 @@ psf_open_handle (PSF_FILE * pfile)
 				return NULL ;
 		} ;
 
+#if defined (WINAPI_FAMILY_PARTITION) && !WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_DESKTOP | WINAPI_PARTITION_SYSTEM)
+	if (!pfile->use_wchar)
+		return NULL ;
+
+	CREATEFILE2_EXTENDED_PARAMETERS cfParams = { 0 } ;
+	cfParams.dwSize = sizeof (CREATEFILE2_EXTENDED_PARAMETERS) ;
+	cfParams.dwFileAttributes = FILE_ATTRIBUTE_NORMAL ;
+
+	handle = CreateFile2 (pfile->path.wc, dwDesiredAccess, dwShareMode, dwCreationDistribution, &cfParams) ;
+
+	if (handle == INVALID_HANDLE_VALUE)
+		return NULL ;
+
+	return handle ;
+#else
 	if (pfile->use_wchar)
 		handle = CreateFileW (
 					pfile->path.wc,				/* pointer to name of the file */
@@ -792,7 +814,7 @@ psf_open_handle (PSF_FILE * pfile)
 					NULL						/* handle to file with attributes to copy */
 					) ;
 	else
-		handle = CreateFile (
+		handle = CreateFileA (
 					pfile->path.c,				/* pointer to name of the file */
 					dwDesiredAccess,			/* access (read-write) mode */
 					dwShareMode,				/* share mode */
@@ -806,6 +828,7 @@ psf_open_handle (PSF_FILE * pfile)
 		return NULL ;
 
 	return handle ;
+#endif
 } /* psf_open_handle */
 
 /* USE_WINDOWS_API */ static void
@@ -896,9 +919,10 @@ psf_file_valid (SF_PRIVATE *psf)
 /* USE_WINDOWS_API */ sf_count_t
 psf_fseek (SF_PRIVATE *psf, sf_count_t offset, int whence)
 {	sf_count_t new_position ;
-	LONG lDistanceToMove, lDistanceToMoveHigh ;
+	LARGE_INTEGER liDistanceToMove, liNewFilePointer ;
 	DWORD dwMoveMethod ;
-	DWORD dwResult, dwError ;
+	BOOL fResult ;
+	DWORD dwError ;
 
 	if (psf->virtual_io)
 		return psf->vio.seek (offset, whence, psf->vio_user_data) ;
@@ -918,12 +942,11 @@ psf_fseek (SF_PRIVATE *psf, sf_count_t offset, int whence)
 				break ;
 		} ;
 
-	lDistanceToMove = (DWORD) (offset & 0xFFFFFFFF) ;
-	lDistanceToMoveHigh = (DWORD) ((offset >> 32) & 0xFFFFFFFF) ;
+	liDistanceToMove.QuadPart = offset ;
 
-	dwResult = SetFilePointer (psf->file.handle, lDistanceToMove, &lDistanceToMoveHigh, dwMoveMethod) ;
+	fResult = SetFilePointerEx (psf->file.handle, liDistanceToMove, &liNewFilePointer, dwMoveMethod) ;
 
-	if (dwResult == 0xFFFFFFFF)
+	if (fResult == FALSE)
 		dwError = GetLastError () ;
 	else
 		dwError = NO_ERROR ;
@@ -933,7 +956,7 @@ psf_fseek (SF_PRIVATE *psf, sf_count_t offset, int whence)
 		return -1 ;
 		} ;
 
-	new_position = (dwResult + ((__int64) lDistanceToMoveHigh << 32)) - psf->fileoffset ;
+	new_position = liNewFilePointer.QuadPart - psf->fileoffset ;
 
 	return new_position ;
 } /* psf_fseek */
@@ -980,7 +1003,7 @@ psf_fread (void *ptr, sf_count_t bytes, sf_count_t items, SF_PRIVATE *psf)
 /* USE_WINDOWS_API */ sf_count_t
 psf_fwrite (const void *ptr, sf_count_t bytes, sf_count_t items, SF_PRIVATE *psf)
 {	sf_count_t total = 0 ;
-	ssize_t	 count ;
+	ssize_t	count ;
 	DWORD dwNumberOfBytesWritten ;
 
 	if (psf->virtual_io)
@@ -1019,8 +1042,9 @@ psf_fwrite (const void *ptr, sf_count_t bytes, sf_count_t items, SF_PRIVATE *psf
 /* USE_WINDOWS_API */ sf_count_t
 psf_ftell (SF_PRIVATE *psf)
 {	sf_count_t pos ;
-	LONG lDistanceToMoveLow, lDistanceToMoveHigh ;
-	DWORD dwResult, dwError ;
+	LARGE_INTEGER liDistanceToMove, liNewFilePointer ;
+	BOOL fResult ;
+	DWORD dwError ;
 
 	if (psf->virtual_io)
 		return psf->vio.tell (psf->vio_user_data) ;
@@ -1028,12 +1052,11 @@ psf_ftell (SF_PRIVATE *psf)
 	if (psf->is_pipe)
 		return psf->pipeoffset ;
 
-	lDistanceToMoveLow = 0 ;
-	lDistanceToMoveHigh = 0 ;
+	liDistanceToMove.QuadPart = 0 ;
 
-	dwResult = SetFilePointer (psf->file.handle, lDistanceToMoveLow, &lDistanceToMoveHigh, FILE_CURRENT) ;
+	fResult = SetFilePointerEx (psf->file.handle, liDistanceToMove, &liNewFilePointer, FILE_CURRENT) ;
 
-	if (dwResult == 0xFFFFFFFF)
+	if (fResult == FALSE)
 		dwError = GetLastError () ;
 	else
 		dwError = NO_ERROR ;
@@ -1043,7 +1066,7 @@ psf_ftell (SF_PRIVATE *psf)
 		return -1 ;
 		} ;
 
-	pos = (dwResult + ((__int64) lDistanceToMoveHigh << 32)) ;
+	pos = liNewFilePointer.QuadPart ;
 
 	return pos - psf->fileoffset ;
 } /* psf_ftell */
@@ -1099,17 +1122,19 @@ psf_is_pipe (SF_PRIVATE *psf)
 /* USE_WINDOWS_API */ sf_count_t
 psf_get_filelen_handle (HANDLE handle)
 {	sf_count_t filelen ;
-	DWORD dwFileSizeLow, dwFileSizeHigh, dwError = NO_ERROR ;
+	LARGE_INTEGER liFileSize ;
+	BOOL fResult ;
+	DWORD dwError = NO_ERROR ;
 
-	dwFileSizeLow = GetFileSize (handle, &dwFileSizeHigh) ;
+	fResult = GetFileSizeEx (handle, &liFileSize) ;
 
-	if (dwFileSizeLow == 0xFFFFFFFF)
+	if (fResult == FALSE)
 		dwError = GetLastError () ;
 
 	if (dwError != NO_ERROR)
 		return (sf_count_t) -1 ;
 
-	filelen = dwFileSizeLow + ((__int64) dwFileSizeHigh << 32) ;
+	filelen = liFileSize.QuadPart ;
 
 	return filelen ;
 } /* psf_get_filelen_handle */
@@ -1123,8 +1148,9 @@ psf_fsync (SF_PRIVATE *psf)
 /* USE_WINDOWS_API */ int
 psf_ftruncate (SF_PRIVATE *psf, sf_count_t len)
 {	int retval = 0 ;
-	LONG lDistanceToMoveLow, lDistanceToMoveHigh ;
-	DWORD dwResult, dwError = NO_ERROR ;
+	LARGE_INTEGER liDistanceToMove ;
+	BOOL fResult ;
+	DWORD dwError = NO_ERROR ;
 
 	/* This implementation trashes the current file position.
 	** should it save and restore it? what if the current position is past
@@ -1135,12 +1161,11 @@ psf_ftruncate (SF_PRIVATE *psf, sf_count_t len)
 	if (len < 0)
 		return 1 ;
 
-	lDistanceToMoveLow = (DWORD) (len & 0xFFFFFFFF) ;
-	lDistanceToMoveHigh = (DWORD) ((len >> 32) & 0xFFFFFFFF) ;
+	liDistanceToMove.QuadPart = (sf_count_t) len ;
 
-	dwResult = SetFilePointer (psf->file.handle, lDistanceToMoveLow, &lDistanceToMoveHigh, FILE_BEGIN) ;
+	fResult = SetFilePointerEx (psf->file.handle, liDistanceToMove, NULL, FILE_BEGIN) ;
 
-	if (dwResult == 0xFFFFFFFF)
+	if (fResult == FALSE)
 		dwError = GetLastError () ;
 
 	if (dwError != NO_ERROR)
@@ -1276,7 +1301,7 @@ psf_fseek (SF_PRIVATE *psf, sf_count_t offset, int whence)
 /* Win32 */ sf_count_t
 psf_fread (void *ptr, sf_count_t bytes, sf_count_t items, SF_PRIVATE *psf)
 {	sf_count_t total = 0 ;
-	ssize_t	 count ;
+	ssize_t	count ;
 
 	if (psf->virtual_io)
 		return psf->vio.read (ptr, bytes*items, psf->vio_user_data) / bytes ;
@@ -1314,7 +1339,7 @@ psf_fread (void *ptr, sf_count_t bytes, sf_count_t items, SF_PRIVATE *psf)
 /* Win32 */ sf_count_t
 psf_fwrite (const void *ptr, sf_count_t bytes, sf_count_t items, SF_PRIVATE *psf)
 {	sf_count_t total = 0 ;
-	ssize_t	 count ;
+	ssize_t	count ;
 
 	if (psf->virtual_io)
 		return psf->vio.write (ptr, bytes*items, psf->vio_user_data) / bytes ;
