@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 1999-2015 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 1999-2017 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -438,7 +438,7 @@ static void
 f2s_array (const float *src, int count, short *dest, float scale)
 {
 	while (--count >= 0)
-	{	dest [count] = lrintf (scale * src [count]) ;
+	{	dest [count] = psf_lrintf (scale * src [count]) ;
 		} ;
 } /* f2s_array */
 
@@ -452,14 +452,14 @@ f2s_clip_array (const float *src, int count, short *dest, float scale)
 		else if (CPU_CLIPS_NEGATIVE == 0 && tmp < -32768.0)
 			dest [count] = SHRT_MIN ;
 		else
-			dest [count] = lrintf (tmp) ;
+			dest [count] = psf_lrintf (tmp) ;
 		} ;
 } /* f2s_clip_array */
 
 static inline void
 f2i_array (const float *src, int count, int *dest, float scale)
 {	while (--count >= 0)
-	{	dest [count] = lrintf (scale * src [count]) ;
+	{	dest [count] = psf_lrintf (scale * src [count]) ;
 		} ;
 } /* f2i_array */
 
@@ -473,7 +473,7 @@ f2i_clip_array (const float *src, int count, int *dest, float scale)
 		else if (CPU_CLIPS_NEGATIVE == 0 && tmp < (-1.0 * INT_MAX))
 			dest [count] = INT_MIN ;
 		else
-			dest [count] = lrintf (tmp) ;
+			dest [count] = psf_lrintf (tmp) ;
 		} ;
 } /* f2i_clip_array */
 
@@ -527,7 +527,7 @@ host_read_f2s	(SF_PRIVATE *psf, short *ptr, sf_count_t len)
 
 /* Fix me : Need lef2s_array */
 		if (psf->data_endswap == SF_TRUE)
-			endswap_int_array (ubuf.ibuf, bufferlen) ;
+			endswap_int_array (ubuf.ibuf, readcount) ;
 
 		convert (ubuf.fbuf, readcount, ptr + total, scale) ;
 		total += readcount ;
@@ -549,7 +549,7 @@ host_read_f2i	(SF_PRIVATE *psf, int *ptr, sf_count_t len)
 
 	convert = (psf->add_clipping) ? f2i_clip_array : f2i_array ;
 	bufferlen = ARRAY_LEN (ubuf.fbuf) ;
-	scale = (psf->float_int_mult == 0) ? 1.0 : 0x7FFFFFFF / psf->float_max ;
+	scale = (psf->float_int_mult == 0) ? 1.0 : 2147483648.0f / psf->float_max ;
 
 	while (len > 0)
 	{	if (len < bufferlen)
