@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 1999-2009 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 1999-2017 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -118,17 +118,12 @@ static char bad_header [] =
 
 	static int
 nist_read_header (SF_PRIVATE *psf)
-{	char	*psf_header ;
+{	char	psf_header [NIST_HEADER_LENGTH + 2] ;
 	int		bitwidth = 0, count, encoding ;
 	unsigned bytes = 0 ;
 	char 	str [64], *cptr ;
 	long	samples ;
 
-	psf_header = psf->u.cbuf ;
-
-	if (sizeof (psf->header) <= NIST_HEADER_LENGTH)
-		return SFE_INTERNAL ;
-
 	/* Go to start of file and read in the whole header. */
 	psf_binheader_readf (psf, "pb", 0, psf_header, NIST_HEADER_LENGTH) ;
 
@@ -201,7 +196,14 @@ nist_read_header (SF_PRIVATE *psf)
 			psf_log_printf (psf, "Weird sample_byte_format : strlen '%s' != %d\n", str, bytes) ;
 
 		if (bytes > 1)
-		{	if (strcmp (str, "01") == 0)
+		{	if (psf->bytewidth == 0)
+				psf->bytewidth = bytes ;
+			else if (psf->bytewidth - bytes != 0)
+			{	psf_log_printf (psf, "psf->bytewidth (%d) != bytes (%d)\n", psf->bytewidth, bytes) ;
+				return SFE_NIST_BAD_ENCODING ;
+				} ;
+
+			if (strcmp (str, "01") == 0)
 				psf->endian = SF_ENDIAN_LITTLE ;
 			else if (strcmp (str, "10") == 0)
 				psf->endian = SF_ENDIAN_BIG ;
@@ -309,8 +311,8 @@ nist_write_header (SF_PRIVATE *psf, int calc_length)
 		end_str = "error" ;
 
 	/* Clear the whole header. */
-	memset (psf->header, 0, sizeof (psf->header)) ;
-	psf->headindex = 0 ;
+	memset (psf->header.ptr, 0, psf->header.len) ;
+	psf->header.indx = 0 ;
 
 	psf_fseek (psf, 0, SEEK_SET) ;
 
@@ -355,9 +357,9 @@ nist_write_header (SF_PRIVATE *psf, int calc_length)
 	psf_asciiheader_printf (psf, "end_head\n") ;
 
 	/* Zero fill to dataoffset. */
-	psf_binheader_writef (psf, "z", (size_t) (NIST_HEADER_LENGTH - psf->headindex)) ;
+	psf_binheader_writef (psf, "z", BHWz ((size_t) (NIST_HEADER_LENGTH - psf->header.indx))) ;
 
-	psf_fwrite (psf->header, psf->headindex, 1, psf) ;
+	psf_fwrite (psf->header.ptr, psf->header.indx, 1, psf) ;
 
 	if (psf->error)
 		return psf->error ;
