@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2001-2009 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 2001-2012 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -25,13 +25,15 @@
 
 #if HAVE_UNISTD_H
 #include <unistd.h>
+#else
+#include "sf_unistd.h"
 #endif
 
 #include <sndfile.h>
 
 #include "utils.h"
 
-#define	BUFFER_LEN		(1<<16)
+#define	BUFFER_LEN		(1 << 16)
 
 static	void	stdin_test	(int typemajor, int count) ;
 
@@ -123,9 +125,9 @@ main (int argc, char *argv [])
 		} ;
 
 	if (test_count == 0)
-	{	fprintf (stderr, "************************************\n") ;
-		fprintf (stderr, "*  No '%s' test defined.\n", argv [1]) ;
-		fprintf (stderr, "************************************\n") ;
+	{	fprintf (stderr, "\n*****************************************\n") ;
+		fprintf (stderr, "*  stdin_test : No '%s' test defined.\n", argv [1]) ;
+		fprintf (stderr, "*****************************************\n") ;
 		return 1 ;
 		} ;
 
@@ -149,7 +151,7 @@ stdin_test	(int typemajor, int count)
 	else
 		memset (&sfinfo, 0, sizeof (sfinfo)) ;
 
-	if ((file = sf_open_fd (STDIN_FILENO, SFM_READ, &sfinfo, SF_TRUE)) == NULL)
+	if ((file = sf_open_fd (fileno (stdin), SFM_READ, &sfinfo, SF_TRUE)) == NULL)
 	{	fprintf (stderr, "sf_open_fd failed with error : ") ;
 		puts (sf_strerror (NULL)) ;
 		dump_log_buffer (NULL) ;
