@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2001-2009 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 2001-2011 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -29,6 +29,8 @@
 
 #if HAVE_UNISTD_H
 #include <unistd.h>
+#else
+#include "sf_unistd.h"
 #endif
 
 #include <sys/types.h>
@@ -61,7 +63,6 @@ main (void)
 
 
 static size_t	file_length (const char *filename) ;
-static int		file_exists (const char *filename) ;
 static void		stdio_test (const char *filetype) ;
 
 static const char *filetypes [] =
@@ -74,9 +75,6 @@ int
 main (void)
 {	int k ;
 
-	if (file_exists ("libsndfile.spec.in"))
-		exit_if_true (chdir ("tests") != 0, "\n    Error : chdir ('tests') failed.\n") ;
-
 	for (k = 0 ; filetypes [k] ; k++)
 		stdio_test (filetypes [k]) ;
 
@@ -92,7 +90,7 @@ stdio_test (const char *filetype)
 
 	print_test_name ("stdio_test", filetype) ;
 
-	snprintf (buffer, sizeof (buffer), "./stdout_test %s > stdio.%s", filetype, filetype) ;
+	snprintf (buffer, sizeof (buffer), "./tests/stdout_test %s > stdio.%s", filetype, filetype) ;
 	if ((retval = system (buffer)))
 	{	retval = WIFEXITED (retval) ? WEXITSTATUS (retval) : 1 ;
 		printf ("%s : %s", buffer, (strerror (retval))) ;
@@ -105,7 +103,7 @@ stdio_test (const char *filetype)
 		exit (1) ;
 		} ;
 
-	snprintf (buffer, sizeof (buffer), "./stdin_test %s < stdio.%s", filetype, filetype) ;
+	snprintf (buffer, sizeof (buffer), "./tests/stdin_test %s < stdio.%s", filetype, filetype) ;
 	if ((retval = system (buffer)))
 	{	retval = WIFEXITED (retval) ? WEXITSTATUS (retval) : 1 ;
 		printf ("%s : %s", buffer, (strerror (retval))) ;
@@ -139,15 +137,5 @@ file_length (const char *filename)
 	return buf.st_size ;
 } /* file_length */
 
-static int
-file_exists (const char *filename)
-{	struct stat buf ;
-
-	if (stat (filename, &buf))
-		return 0 ;
-
-	return 1 ;
-} /* file_exists */
-
 #endif
 
